"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.HMSConfig = void 0;
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == typeof i ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != typeof t || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != typeof i) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
/**
 * Represents the configuration needed to join a room in the HMS ecosystem.
 *
 * This class encapsulates all necessary details required to initiate a connection to a room, including user identification,
 * authentication token, optional endpoint for different server connections, metadata for additional information about the connection,
 * and a flag to capture network quality during the preview phase.
 *
 *
 * @see https://www.100ms.live/docs/react-native/v2/how-to-guides/set-up-video-conferencing/join
 *
 * @param {Object} params - The configuration parameters.
 * @param {string} params.username - The username of the participant.
 * @param {string} params.authToken - The authentication token for room access.
 * @param {string} [params.endpoint] - Optional. The endpoint URL for the connection.
 * @param {string} [params.metadata] - Optional. Additional metadata associated with the Local Peer.
 * @param {boolean} [params.captureNetworkQualityInPreview] - Optional. Flag to capture network quality in the preview phase.
 * @constructor
 *
 * @example
 * const config = new HMSConfig({
 *  username: 'username',
 *  authToken: 'authToken',
 *  metadata: 'metadata',
 *  captureNetworkQualityInPreview: true,
 * });
 *
 */
class HMSConfig {
  constructor(params) {
    _defineProperty(this, "username", void 0);
    _defineProperty(this, "authToken", void 0);
    _defineProperty(this, "endpoint", void 0);
    _defineProperty(this, "metadata", void 0);
    _defineProperty(this, "captureNetworkQualityInPreview", void 0);
    this.username = params.username;
    this.authToken = params.authToken;
    this.endpoint = params.endpoint;
    this.metadata = params.metadata;
    this.captureNetworkQualityInPreview = params.captureNetworkQualityInPreview;
  }
}
exports.HMSConfig = HMSConfig;
//# sourceMappingURL=HMSConfig.js.map