"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.HMSEncoder = void 0;
var _HMSManagerModule = _interopRequireDefault(require("../modules/HMSManagerModule"));
var _HMSTrack = require("./HMSTrack");
var _HMSAudioTrack = require("./HMSAudioTrack");
var _HMSVideoTrack = require("./HMSVideoTrack");
var _HMSRoom = require("./HMSRoom");
var _HMSPeer = require("./HMSPeer");
var _HMSLocalPeer = require("./HMSLocalPeer");
var _HMSRemotePeer = require("./HMSRemotePeer");
var _HMSAudioTrackSettings = require("./HMSAudioTrackSettings");
var _HMSVideoTrackSettings = require("./HMSVideoTrackSettings");
var _HMSLocalVideoTrack = require("./HMSLocalVideoTrack");
var _HMSLocalAudioTrack = require("./HMSLocalAudioTrack");
var _HMSRole = require("./HMSRole");
var _HMSRoleChangeRequest = require("./HMSRoleChangeRequest");
var _HMSChangeTrackStateRequest = require("./HMSChangeTrackStateRequest");
var _HMSVideoResolution = require("./HMSVideoResolution");
var _HMSRTCStats = require("./HMSRTCStats");
var _HMSRTCStatsReport = require("./HMSRTCStatsReport");
var _HMSRemoteAudioTrack = require("./HMSRemoteAudioTrack");
var _HMSRemoteVideoTrack = require("./HMSRemoteVideoTrack");
var _HMSSpeaker = require("./HMSSpeaker");
var _HMSHLSRecordingState = require("./HMSHLSRecordingState");
var _HMSNetworkQuality = require("./HMSNetworkQuality");
var _HMSBrowserRecordingState = require("./HMSBrowserRecordingState");
var _HMSHLSStreamingState = require("./HMSHLSStreamingState");
var _HMSHLSVariant = require("./HMSHLSVariant");
var _HMSRtmpStreamingState = require("./HMSRtmpStreamingState");
var _HMSServerRecordingState = require("./HMSServerRecordingState");
var _HMSMessage = require("./HMSMessage");
var _HMSMessageRecipient = require("./HMSMessageRecipient");
var _HMSException = require("./HMSException");
var _HMSConstants = require("./HMSConstants");
var _HMSPeerUpdate = require("./HMSPeerUpdate");
var _HMSLocalAudioStats = require("./HMSLocalAudioStats");
var _HMSLocalVideoStats = require("./HMSLocalVideoStats");
var _HMSRemoteAudioStats = require("./HMSRemoteAudioStats");
var _HMSRemoteVideoStats = require("./HMSRemoteVideoStats");
var _HMSLayer = require("./HMSLayer");
var _HMSSimulcastLayerDefinition = require("./HMSSimulcastLayerDefinition");
var _HMSQualityLimitationReasons = require("./HMSQualityLimitationReasons");
var _HMSTrackType = require("./HMSTrackType");
var _HMSHLSPlaylistType = require("./HMSHLSPlaylistType");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == typeof i ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != typeof t || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != typeof i) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
class HMSEncoder {
  static clearData() {
    this.data = {
      roles: {}
    };
  }
  static encodeHmsRoom(room, id) {
    const encodedObj = {
      id: room === null || room === void 0 ? void 0 : room.id,
      sessionId: (room === null || room === void 0 ? void 0 : room.sessionId) || undefined,
      metaData: room === null || room === void 0 ? void 0 : room.metaData,
      name: room === null || room === void 0 ? void 0 : room.name,
      peerCount: room === null || room === void 0 ? void 0 : room.peerCount,
      peers: HMSEncoder.encodeHmsPeers(room === null || room === void 0 ? void 0 : room.peers),
      browserRecordingState: HMSEncoder.encodeBrowserRecordingState(room === null || room === void 0 ? void 0 : room.browserRecordingState),
      rtmpHMSRtmpStreamingState: HMSEncoder.encodeRTMPStreamingState(room === null || room === void 0 ? void 0 : room.rtmpHMSRtmpStreamingState),
      serverRecordingState: HMSEncoder.encodeServerRecordingState(room === null || room === void 0 ? void 0 : room.serverRecordingState),
      hlsStreamingState: HMSEncoder.encodeHLSStreamingState(room === null || room === void 0 ? void 0 : room.hlsStreamingState),
      hlsRecordingState: HMSEncoder.encodeHLSRecordingState(room === null || room === void 0 ? void 0 : room.hlsRecordingState),
      localPeer: HMSEncoder.encodeHmsLocalPeer(room === null || room === void 0 ? void 0 : room.localPeer, id)
      // startedAt: HMSEncoder.encodeDate(room?.startedAt),
    };
    return new _HMSRoom.HMSRoom(encodedObj);
  }
  static encodeHmsPeers(peers) {
    const encodedPeers = [];
    peers === null || peers === void 0 || peers.map(peer => {
      encodedPeers.push(HMSEncoder.encodeHmsPeer(peer));
    });
    return encodedPeers;
  }
  static encodeHmsPeer(peer) {
    const encodedObj = {
      peerID: peer === null || peer === void 0 ? void 0 : peer.peerID,
      name: peer === null || peer === void 0 ? void 0 : peer.name
    };
    return new _HMSPeer.HMSPeer(encodedObj);
  }
  static encodeHmsPeerUpdate(hmsPeerUpdateOrdinal) {
    return _HMSPeerUpdate.HMSPeerUpdateOrdinals.get(hmsPeerUpdateOrdinal);
  }
  static encodeHmsAudioTrack(track, id) {
    const encodedObj = {
      trackId: track === null || track === void 0 ? void 0 : track.trackId,
      source: track === null || track === void 0 ? void 0 : track.source,
      trackDescription: track === null || track === void 0 ? void 0 : track.trackDescription,
      isMute: track === null || track === void 0 ? void 0 : track.isMute,
      id: id,
      type: track === null || track === void 0 ? void 0 : track.type
    };
    return new _HMSAudioTrack.HMSAudioTrack(encodedObj);
  }
  static encodeHmsVideoTrack(track, id) {
    const encodedObj = {
      id: id,
      trackId: track === null || track === void 0 ? void 0 : track.trackId,
      source: track === null || track === void 0 ? void 0 : track.source,
      trackDescription: track === null || track === void 0 ? void 0 : track.trackDescription,
      isMute: track === null || track === void 0 ? void 0 : track.isMute,
      isDegraded: track === null || track === void 0 ? void 0 : track.isDegraded,
      type: track === null || track === void 0 ? void 0 : track.type
    };
    return new _HMSVideoTrack.HMSVideoTrack(encodedObj);
  }
  static encodeHmsAuxiliaryTracks(tracks, id) {
    const auxiliaryTracks = [];
    tracks === null || tracks === void 0 || tracks.map(track => {
      auxiliaryTracks.push(HMSEncoder.encodeHmsTrack(track, id));
    });
    return auxiliaryTracks;
  }
  static encodeHmsTrack(track, id) {
    const encodedObj = {
      trackId: track === null || track === void 0 ? void 0 : track.trackId,
      source: track === null || track === void 0 ? void 0 : track.source,
      trackDescription: track === null || track === void 0 ? void 0 : track.trackDescription,
      isMute: track === null || track === void 0 ? void 0 : track.isMute,
      id: id,
      type: track === null || track === void 0 ? void 0 : track.type
    };
    return new _HMSTrack.HMSTrack(encodedObj);
  }
  static encodeHmsLocalPeer(peer, id) {
    var _peer$localAudioTrack, _peer$localAudioTrack2, _peer$localAudioTrack3, _peer$localAudioTrack4, _peer$localAudioTrack5, _peer$localAudioTrack6, _peer$localAudioTrack7, _peer$localAudioTrack8, _peer$localVideoTrack, _peer$localVideoTrack2, _peer$localVideoTrack3, _peer$localVideoTrack4, _peer$localVideoTrack5, _peer$localVideoTrack6, _peer$localVideoTrack7, _peer$localVideoTrack8;
    const encodedObj = {
      peerID: peer === null || peer === void 0 ? void 0 : peer.peerID,
      name: peer === null || peer === void 0 ? void 0 : peer.name,
      localAudioTrackData: peer !== null && peer !== void 0 && (_peer$localAudioTrack = peer.localAudioTrackData) !== null && _peer$localAudioTrack !== void 0 && _peer$localAudioTrack.trackId ? {
        id: id,
        trackId: peer === null || peer === void 0 || (_peer$localAudioTrack2 = peer.localAudioTrackData) === null || _peer$localAudioTrack2 === void 0 ? void 0 : _peer$localAudioTrack2.trackId,
        source: peer === null || peer === void 0 || (_peer$localAudioTrack3 = peer.localAudioTrackData) === null || _peer$localAudioTrack3 === void 0 ? void 0 : _peer$localAudioTrack3.source,
        trackDescription: peer === null || peer === void 0 || (_peer$localAudioTrack4 = peer.localAudioTrackData) === null || _peer$localAudioTrack4 === void 0 ? void 0 : _peer$localAudioTrack4.trackDescription,
        isMute: peer === null || peer === void 0 || (_peer$localAudioTrack5 = peer.localAudioTrackData) === null || _peer$localAudioTrack5 === void 0 ? void 0 : _peer$localAudioTrack5.isMute,
        settings: peer !== null && peer !== void 0 && (_peer$localAudioTrack6 = peer.localAudioTrackData) !== null && _peer$localAudioTrack6 !== void 0 && _peer$localAudioTrack6.settings ? HMSEncoder.encodeHmsAudioTrackSettings(peer === null || peer === void 0 || (_peer$localAudioTrack7 = peer.localAudioTrackData) === null || _peer$localAudioTrack7 === void 0 ? void 0 : _peer$localAudioTrack7.settings) : undefined,
        type: peer === null || peer === void 0 || (_peer$localAudioTrack8 = peer.localAudioTrackData) === null || _peer$localAudioTrack8 === void 0 ? void 0 : _peer$localAudioTrack8.type
      } : undefined,
      localVideoTrackData: peer !== null && peer !== void 0 && (_peer$localVideoTrack = peer.localVideoTrackData) !== null && _peer$localVideoTrack !== void 0 && _peer$localVideoTrack.trackId ? {
        id: id,
        trackId: peer === null || peer === void 0 || (_peer$localVideoTrack2 = peer.localVideoTrackData) === null || _peer$localVideoTrack2 === void 0 ? void 0 : _peer$localVideoTrack2.trackId,
        source: peer === null || peer === void 0 || (_peer$localVideoTrack3 = peer.localVideoTrackData) === null || _peer$localVideoTrack3 === void 0 ? void 0 : _peer$localVideoTrack3.source,
        trackDescription: peer === null || peer === void 0 || (_peer$localVideoTrack4 = peer.localVideoTrackData) === null || _peer$localVideoTrack4 === void 0 ? void 0 : _peer$localVideoTrack4.trackDescription,
        isMute: peer === null || peer === void 0 || (_peer$localVideoTrack5 = peer.localVideoTrackData) === null || _peer$localVideoTrack5 === void 0 ? void 0 : _peer$localVideoTrack5.isMute,
        settings: peer !== null && peer !== void 0 && (_peer$localVideoTrack6 = peer.localVideoTrackData) !== null && _peer$localVideoTrack6 !== void 0 && _peer$localVideoTrack6.settings ? HMSEncoder.encodeHmsVideoTrackSettings(peer === null || peer === void 0 || (_peer$localVideoTrack7 = peer.localVideoTrackData) === null || _peer$localVideoTrack7 === void 0 ? void 0 : _peer$localVideoTrack7.settings) : undefined,
        type: peer === null || peer === void 0 || (_peer$localVideoTrack8 = peer.localVideoTrackData) === null || _peer$localVideoTrack8 === void 0 ? void 0 : _peer$localVideoTrack8.type
      } : undefined
    };
    return new _HMSLocalPeer.HMSLocalPeer(encodedObj);
  }
  static encodeHmsAudioTrackSettings(settings) {
    const encodedObj = {
      useHardwareEchoCancellation: settings === null || settings === void 0 ? void 0 : settings.useHardwareAcousticEchoCanceler,
      initialState: settings === null || settings === void 0 ? void 0 : settings.initialState,
      audioMode: settings === null || settings === void 0 ? void 0 : settings.audioMode
    };
    return new _HMSAudioTrackSettings.HMSAudioTrackSettings(encodedObj);
  }
  static encodeHmsVideoTrackSettings(settings) {
    const encodedObj = {
      initialState: settings === null || settings === void 0 ? void 0 : settings.initialState,
      forceSoftwareDecoder: settings === null || settings === void 0 ? void 0 : settings.forceSoftwareDecoder,
      simulcastSettings: settings === null || settings === void 0 ? void 0 : settings.simulcastSettings,
      cameraFacing: settings === null || settings === void 0 ? void 0 : settings.cameraFacing,
      disableAutoResize: settings === null || settings === void 0 ? void 0 : settings.disableAutoResize
    };
    return new _HMSVideoTrackSettings.HMSVideoTrackSettings(encodedObj);
  }
  static encodeHmsVideoResolution(resolution) {
    const encodedObj = {
      height: resolution === null || resolution === void 0 ? void 0 : resolution.height,
      width: resolution === null || resolution === void 0 ? void 0 : resolution.width
    };
    return new _HMSVideoResolution.HMSVideoResolution(encodedObj);
  }
  static encodeHmsLocalAudioTrack(track, id) {
    const encodedObj = {
      id: id,
      trackId: track === null || track === void 0 ? void 0 : track.trackId,
      source: track === null || track === void 0 ? void 0 : track.source,
      trackDescription: track === null || track === void 0 ? void 0 : track.trackDescription,
      isMute: track === null || track === void 0 ? void 0 : track.isMute,
      settings: track !== null && track !== void 0 && track.settings ? HMSEncoder.encodeHmsAudioTrackSettings(track === null || track === void 0 ? void 0 : track.settings) : undefined,
      type: track === null || track === void 0 ? void 0 : track.type
    };
    return new _HMSLocalAudioTrack.HMSLocalAudioTrack(encodedObj);
  }
  static encodeHmsLocalVideoTrack(track, id) {
    const encodedObj = {
      id: id,
      trackId: track === null || track === void 0 ? void 0 : track.trackId,
      source: track === null || track === void 0 ? void 0 : track.source,
      trackDescription: track === null || track === void 0 ? void 0 : track.trackDescription,
      isMute: track === null || track === void 0 ? void 0 : track.isMute,
      settings: track !== null && track !== void 0 && track.settings ? HMSEncoder.encodeHmsVideoTrackSettings(track === null || track === void 0 ? void 0 : track.settings) : undefined,
      type: track === null || track === void 0 ? void 0 : track.type
    };
    return new _HMSLocalVideoTrack.HMSLocalVideoTrack(encodedObj);
  }
  static encodeHmsRemotePeers(peers, id) {
    const hmsPeers = [];
    peers.map(peer => {
      const encodedPeer = HMSEncoder.encodeHmsRemotePeer(peer, id);
      hmsPeers.push(encodedPeer);
    });
    return hmsPeers;
  }
  static encodeHmsRemotePeer(peer, id) {
    var _peer$remoteAudioTrac, _peer$remoteAudioTrac2, _peer$remoteAudioTrac3, _peer$remoteAudioTrac4, _peer$remoteAudioTrac5, _peer$remoteAudioTrac6, _peer$remoteVideoTrac, _peer$remoteVideoTrac2, _peer$remoteVideoTrac3, _peer$remoteVideoTrac4, _peer$remoteVideoTrac5, _peer$remoteVideoTrac6;
    const encodedObj = {
      peerID: peer === null || peer === void 0 ? void 0 : peer.peerID,
      name: peer === null || peer === void 0 ? void 0 : peer.name,
      remoteAudioTrackData: peer !== null && peer !== void 0 && (_peer$remoteAudioTrac = peer.remoteAudioTrackData) !== null && _peer$remoteAudioTrac !== void 0 && _peer$remoteAudioTrac.trackId ? {
        id: id,
        trackId: peer === null || peer === void 0 || (_peer$remoteAudioTrac2 = peer.remoteAudioTrackData) === null || _peer$remoteAudioTrac2 === void 0 ? void 0 : _peer$remoteAudioTrac2.trackId,
        source: peer === null || peer === void 0 || (_peer$remoteAudioTrac3 = peer.remoteAudioTrackData) === null || _peer$remoteAudioTrac3 === void 0 ? void 0 : _peer$remoteAudioTrac3.source,
        trackDescription: peer === null || peer === void 0 || (_peer$remoteAudioTrac4 = peer.remoteAudioTrackData) === null || _peer$remoteAudioTrac4 === void 0 ? void 0 : _peer$remoteAudioTrac4.trackDescription,
        isMute: peer === null || peer === void 0 || (_peer$remoteAudioTrac5 = peer.remoteAudioTrackData) === null || _peer$remoteAudioTrac5 === void 0 ? void 0 : _peer$remoteAudioTrac5.isMute,
        playbackAllowed: peer === null || peer === void 0 || (_peer$remoteAudioTrac6 = peer.remoteAudioTrackData) === null || _peer$remoteAudioTrac6 === void 0 ? void 0 : _peer$remoteAudioTrac6.playbackAllowed
      } : undefined,
      remoteVideoTrackData: peer !== null && peer !== void 0 && (_peer$remoteVideoTrac = peer.remoteVideoTrackData) !== null && _peer$remoteVideoTrac !== void 0 && _peer$remoteVideoTrac.trackId ? {
        id: id,
        trackId: peer === null || peer === void 0 || (_peer$remoteVideoTrac2 = peer.remoteVideoTrackData) === null || _peer$remoteVideoTrac2 === void 0 ? void 0 : _peer$remoteVideoTrac2.trackId,
        source: peer === null || peer === void 0 || (_peer$remoteVideoTrac3 = peer.remoteVideoTrackData) === null || _peer$remoteVideoTrac3 === void 0 ? void 0 : _peer$remoteVideoTrac3.source,
        trackDescription: peer === null || peer === void 0 || (_peer$remoteVideoTrac4 = peer.remoteVideoTrackData) === null || _peer$remoteVideoTrac4 === void 0 ? void 0 : _peer$remoteVideoTrac4.trackDescription,
        isMute: peer === null || peer === void 0 || (_peer$remoteVideoTrac5 = peer.remoteVideoTrackData) === null || _peer$remoteVideoTrac5 === void 0 ? void 0 : _peer$remoteVideoTrac5.isMute,
        playbackAllowed: peer === null || peer === void 0 || (_peer$remoteVideoTrac6 = peer.remoteVideoTrackData) === null || _peer$remoteVideoTrac6 === void 0 ? void 0 : _peer$remoteVideoTrac6.playbackAllowed
      } : undefined
    };
    return new _HMSRemotePeer.HMSRemotePeer(encodedObj);
  }
  static encodeHmsRemoteAudioTrack(track, id) {
    const encodedObj = {
      id: id,
      trackId: track === null || track === void 0 ? void 0 : track.trackId,
      source: track === null || track === void 0 ? void 0 : track.source,
      trackDescription: track === null || track === void 0 ? void 0 : track.trackDescription,
      isMute: track === null || track === void 0 ? void 0 : track.isMute,
      playbackAllowed: track === null || track === void 0 ? void 0 : track.playbackAllowed
    };
    return new _HMSRemoteAudioTrack.HMSRemoteAudioTrack(encodedObj);
  }
  static encodeHmsRemoteVideoTrack(track, id) {
    const encodedObj = {
      id: id,
      trackId: track === null || track === void 0 ? void 0 : track.trackId,
      source: track === null || track === void 0 ? void 0 : track.source,
      trackDescription: track === null || track === void 0 ? void 0 : track.trackDescription,
      isMute: track === null || track === void 0 ? void 0 : track.isMute,
      playbackAllowed: track === null || track === void 0 ? void 0 : track.playbackAllowed
    };
    return new _HMSRemoteVideoTrack.HMSRemoteVideoTrack(encodedObj);
  }
  static encodeHmsPreviewTracks(previewTracks, id) {
    return previewTracks === null || previewTracks === void 0 ? void 0 : previewTracks.map(track => {
      return this.encodeHmsTrack(track, id);
    });
  }
  static encodeHmsPreviewForRoleTracks(previewTracks, id) {
    return previewTracks === null || previewTracks === void 0 ? void 0 : previewTracks.map(track => {
      return track.type === _HMSTrackType.HMSTrackType.VIDEO ? this.encodeHmsLocalVideoTrack(track, id) : this.encodeHmsLocalAudioTrack(track, id);
    });
  }
  static encodeHmsRoles(roles) {
    const encodedRoles = [];
    roles === null || roles === void 0 || roles.map(item => {
      encodedRoles.push(HMSEncoder.encodeHmsRole(item));
    });
    return encodedRoles;
  }
  static encodeHmsRole(role) {
    var _cachedRole$publishSe;
    if (!role) {
      return new _HMSRole.HMSRole(role);
    }
    const rolesCache = this.data.roles;
    const cachedRole = rolesCache[role.name];

    // create new HMSRole instance, if cached role does not exist OR `role.publishSettings?.allowed` does not exist
    if (!cachedRole || !((_cachedRole$publishSe = cachedRole.publishSettings) !== null && _cachedRole$publishSe !== void 0 && _cachedRole$publishSe.allowed)) {
      var _hmsRole$publishSetti;
      // Creating HMSRole object with data
      const hmsRole = new _HMSRole.HMSRole(role);

      // saving the created HMSRole object into cache
      rolesCache[role.name] = hmsRole;

      // If the created HMSRole object is complete,
      // sending notification to Native Side to stop sending data for this role
      if ((_hmsRole$publishSetti = hmsRole.publishSettings) !== null && _hmsRole$publishSetti !== void 0 && _hmsRole$publishSetti.allowed) {
        _HMSManagerModule.default.restrictData({
          id: _HMSConstants.HMSConstants.DEFAULT_SDK_ID,
          roleName: hmsRole.name
        });
      }
    }
    return rolesCache[role.name];
  }
  static encodeHmsRoleChangeRequest(data) {
    const encodedRoleChangeRequest = {
      requestedBy: data.requestedBy ? HMSEncoder.encodeHmsPeer(data.requestedBy) : undefined,
      suggestedRole: HMSEncoder.encodeHmsRole(data.suggestedRole)
    };
    return new _HMSRoleChangeRequest.HMSRoleChangeRequest(encodedRoleChangeRequest);
  }
  static encodeHmsChangeTrackStateRequest(data) {
    const encodedChangeTrackStateRequest = {
      requestedBy: data !== null && data !== void 0 && data.requestedBy ? HMSEncoder.encodeHmsPeer(data === null || data === void 0 ? void 0 : data.requestedBy) : undefined,
      trackType: data.trackType,
      mute: data.mute
    };
    return new _HMSChangeTrackStateRequest.HMSChangeTrackStateRequest(encodedChangeTrackStateRequest);
  }
  static encodeRTCStats(data) {
    let video = this.encodeRTCStatsUnit(data === null || data === void 0 ? void 0 : data.video);
    let audio = this.encodeRTCStatsUnit(data === null || data === void 0 ? void 0 : data.audio);
    let combined = this.encodeRTCStatsUnit(data === null || data === void 0 ? void 0 : data.combined);
    return new _HMSRTCStatsReport.HMSRTCStatsReport({
      video,
      audio,
      combined
    });
  }
  static encodeRTCStatsUnit(data) {
    const [bitrateReceived, bitrateSent, bytesReceived, bytesSent, packetsLost, packetsReceived, roundTripTime] = data;
    return new _HMSRTCStats.HMSRTCStats({
      bitrateReceived,
      bitrateSent,
      bytesReceived,
      bytesSent,
      packetsLost,
      packetsReceived,
      roundTripTime
    });
  }
  static encodeHmsSpeakers(data, id) {
    let encodedSpeakers = [];
    data === null || data === void 0 || data.map(item => {
      encodedSpeakers.push(HMSEncoder.encodeHmsSpeaker(item, id));
    });
    return encodedSpeakers;
  }
  static encodeHmsSpeaker(data, id) {
    return new _HMSSpeaker.HMSSpeaker({
      level: data === null || data === void 0 ? void 0 : data.level,
      peer: HMSEncoder.encodeHmsPeer(data === null || data === void 0 ? void 0 : data.peer),
      track: HMSEncoder.encodeHmsTrack(data === null || data === void 0 ? void 0 : data.track, id)
    });
  }
  static encodeBrowserRecordingState(data) {
    return new _HMSBrowserRecordingState.HMSBrowserRecordingState({
      initialising: (data === null || data === void 0 ? void 0 : data.initialising) || false,
      running: (data === null || data === void 0 ? void 0 : data.running) || false,
      startedAt: HMSEncoder.encodeDate(data === null || data === void 0 ? void 0 : data.startedAt),
      stoppedAt: HMSEncoder.encodeDate(data === null || data === void 0 ? void 0 : data.stoppedAt),
      error: (data === null || data === void 0 ? void 0 : data.error) || undefined,
      state: data === null || data === void 0 ? void 0 : data.state
    });
  }
  static encodeServerRecordingState(data) {
    return new _HMSServerRecordingState.HMSServerRecordingState({
      running: (data === null || data === void 0 ? void 0 : data.running) || false,
      error: (data === null || data === void 0 ? void 0 : data.error) || undefined,
      startedAt: HMSEncoder.encodeDate(data === null || data === void 0 ? void 0 : data.startedAt),
      state: data === null || data === void 0 ? void 0 : data.state
    });
  }
  static encodeRTMPStreamingState(data) {
    return new _HMSRtmpStreamingState.HMSRtmpStreamingState({
      running: (data === null || data === void 0 ? void 0 : data.running) || false,
      startedAt: HMSEncoder.encodeDate(data === null || data === void 0 ? void 0 : data.startedAt),
      error: (data === null || data === void 0 ? void 0 : data.error) || undefined,
      state: data === null || data === void 0 ? void 0 : data.state
    });
  }
  static encodeDate(dateData) {
    if (!dateData) {
      return undefined;
    }
    const dateNum = parseInt(dateData);
    if (isNaN(dateNum)) {
      return undefined;
    }
    return new Date(dateNum);
  }
  static encodeHLSStreamingState(data) {
    return new _HMSHLSStreamingState.HMSHLSStreamingState({
      running: (data === null || data === void 0 ? void 0 : data.running) || false,
      startedAt: HMSEncoder.encodeDate(data === null || data === void 0 ? void 0 : data.startedAt),
      error: (data === null || data === void 0 ? void 0 : data.error) || undefined,
      state: data === null || data === void 0 ? void 0 : data.state,
      variants: Array.isArray(data === null || data === void 0 ? void 0 : data.variants) ? this.encodeHLSVariants(data === null || data === void 0 ? void 0 : data.variants) : undefined
    });
  }
  static encodeHLSRecordingState(data) {
    if (data) {
      return new _HMSHLSRecordingState.HMSHLSRecordingState({
        running: (data === null || data === void 0 ? void 0 : data.running) || false,
        startedAt: HMSEncoder.encodeDate(data === null || data === void 0 ? void 0 : data.startedAt),
        error: (data === null || data === void 0 ? void 0 : data.error) || undefined,
        state: data === null || data === void 0 ? void 0 : data.state
      });
    } else {
      return undefined;
    }
  }
  static encodeHLSVariants(data) {
    let variants = [];
    data === null || data === void 0 || data.map(item => {
      let variant = new _HMSHLSVariant.HMSHLSVariant({
        hlsStreamUrl: item.hlsStreamUrl,
        meetingUrl: item.meetingUrl,
        metadata: item !== null && item !== void 0 && item.metaData ? item === null || item === void 0 ? void 0 : item.metadata : undefined,
        startedAt: HMSEncoder.encodeDate(item === null || item === void 0 ? void 0 : item.startedAt),
        playlistType: item !== null && item !== void 0 && item.playlistType ? this.getHLSVariantPlaylistType(item.playlistType.toUpperCase()) : undefined
      });
      variants.push(variant);
    });
    return variants;
  }
  static getHLSVariantPlaylistType(type) {
    switch (type) {
      case 'DVR':
        return _HMSHLSPlaylistType.HMSHLSPlaylistType.DVR;
      default:
        return _HMSHLSPlaylistType.HMSHLSPlaylistType.noDVR;
    }
  }
  static encodeHMSNetworkQuality(data) {
    if (data) {
      return new _HMSNetworkQuality.HMSNetworkQuality({
        downlinkQuality: data === null || data === void 0 ? void 0 : data.downlinkQuality
      });
    } else {
      return undefined;
    }
  }
  static encodeHMSMessage(data) {
    if (data) {
      return new _HMSMessage.HMSMessage({
        message: data === null || data === void 0 ? void 0 : data.message,
        messageId: data === null || data === void 0 ? void 0 : data.messageId,
        type: data === null || data === void 0 ? void 0 : data.type,
        time: new Date(parseInt(data === null || data === void 0 ? void 0 : data.time)),
        sender: this.encodeHmsPeer(data === null || data === void 0 ? void 0 : data.sender),
        recipient: this.encodeHMSMessageRecipient(data === null || data === void 0 ? void 0 : data.recipient)
      });
    } else {
      return undefined;
    }
  }
  static encodeHMSMessageRecipient(data) {
    return new _HMSMessageRecipient.HMSMessageRecipient({
      recipientType: data === null || data === void 0 ? void 0 : data.recipientType,
      recipientPeer: data !== null && data !== void 0 && data.recipientPeer ? this.encodeHmsPeer(data.recipientPeer) : undefined,
      recipientRoles: Array.isArray(data === null || data === void 0 ? void 0 : data.recipientRoles) ? this.encodeHmsRoles(data.recipientRoles) : []
    });
  }
  static encodeHMSException(data) {
    var _data$error, _data$error2, _data$error3, _data$error4, _data$error5, _data$error6, _data$error7;
    return new _HMSException.HMSException({
      code: data === null || data === void 0 || (_data$error = data.error) === null || _data$error === void 0 ? void 0 : _data$error.code,
      description: data === null || data === void 0 || (_data$error2 = data.error) === null || _data$error2 === void 0 ? void 0 : _data$error2.description,
      message: data === null || data === void 0 || (_data$error3 = data.error) === null || _data$error3 === void 0 ? void 0 : _data$error3.message,
      name: data === null || data === void 0 || (_data$error4 = data.error) === null || _data$error4 === void 0 ? void 0 : _data$error4.name,
      action: data === null || data === void 0 || (_data$error5 = data.error) === null || _data$error5 === void 0 ? void 0 : _data$error5.action,
      isTerminal: data === null || data === void 0 || (_data$error6 = data.error) === null || _data$error6 === void 0 ? void 0 : _data$error6.isTerminal,
      canRetry: data === null || data === void 0 || (_data$error7 = data.error) === null || _data$error7 === void 0 ? void 0 : _data$error7.canRetry
    });
  }
  static encodeHMSLocalAudioStats(data) {
    const [bitrate, bytesSent, roundTripTime] = data;
    return new _HMSLocalAudioStats.HMSLocalAudioStats({
      bitrate: bitrate >= 0 ? bitrate : undefined,
      bytesSent: bytesSent ? bytesSent : undefined,
      roundTripTime: roundTripTime >= 0 ? roundTripTime : undefined
    });
  }
  static encodeHMSLocalVideoStats(data) {
    return data.map(item => {
      const [bitrate, bytesSent, roundTripTime, frameRate, resolution, layer, qualityLimitationReasons] = item;
      return new _HMSLocalVideoStats.HMSLocalVideoStats({
        bitrate: bitrate >= 0 ? bitrate : undefined,
        bytesSent: bytesSent ? bytesSent : undefined,
        roundTripTime: roundTripTime >= 0 ? roundTripTime : undefined,
        frameRate: frameRate >= 0 ? frameRate : undefined,
        resolution: resolution ? {
          width: resolution[0],
          height: resolution[1]
        } : undefined,
        // resolution: [width, height]
        layer: layer ? layer : undefined,
        qualityLimitationReasons: qualityLimitationReasons ? this.encodeHMSQualityLimitationReasons(qualityLimitationReasons) : undefined
      });
    });
  }
  static encodeHMSRemoteAudioStats(data) {
    const [bitrate, bytesReceived, jitter, packetsLost, packetsReceived] = data;
    return new _HMSRemoteAudioStats.HMSRemoteAudioStats({
      bitrate: bitrate >= 0 ? bitrate : undefined,
      bytesReceived: bytesReceived ? bytesReceived : undefined,
      jitter: jitter >= 0 ? jitter : undefined,
      packetsLost: packetsLost >= 0 ? packetsLost : undefined,
      packetsReceived: packetsReceived ? packetsReceived : undefined
    });
  }
  static encodeHMSRemoteVideoStats(data) {
    const [bitrate, bytesReceived, frameRate, jitter, packetsLost, packetsReceived, resolution] = data;
    return new _HMSRemoteVideoStats.HMSRemoteVideoStats({
      bitrate: bitrate >= 0 ? bitrate : undefined,
      bytesReceived: bytesReceived ? bytesReceived : undefined,
      frameRate: frameRate >= 0 ? frameRate : undefined,
      jitter: jitter >= 0 ? jitter : undefined,
      packetsLost: packetsLost >= 0 ? packetsLost : undefined,
      packetsReceived: packetsReceived ? packetsReceived : undefined,
      resolution: resolution ? {
        width: resolution[0],
        height: resolution[1]
      } : undefined // resolution: [width, height]
    });
  }
  static encodeHMSSimulcastLayerDefinition(data) {
    return data.map(sld => {
      return new _HMSSimulcastLayerDefinition.HMSSimulcastLayerDefinition({
        layer: _HMSLayer.HMSLayer[sld.layer],
        // DOUBT: This can be invalid. Should we throw error?
        resolution: this.encodeHmsVideoResolution({
          width: sld.resolution[0],
          height: sld.resolution[1]
        })
      });
    });
  }
  static encodeHMSQualityLimitationReasons(data) {
    return new _HMSQualityLimitationReasons.HMSQualityLimitationReasons({
      reason: data.reason,
      bandwidth: data.bandwidth,
      cpu: data.cpu,
      none: data.none,
      other: data.other,
      qualityLimitationResolutionChanges: data.qualityLimitationResolutionChanges
    });
  }
  static transformHMSHLSCueEventData(data) {
    if ('startDate' in data) {
      data.startDate = this.encodeDate(data.startDate);
    }
    if ('endDate' in data && data.endDate) {
      data.endDate = this.encodeDate(data.endDate);
    }
    const transformed = data;
    return transformed;
  }
  static encodeTranscriptions(data) {
    if (Array.isArray(data)) {
      data.forEach(transcriptions => {
        if ('initialisedAt' in transcriptions) {
          transcriptions.initialisedAt = this.encodeDate(transcriptions.initialisedAt);
        }
        if ('startedAt' in transcriptions) {
          transcriptions.startedAt = this.encodeDate(transcriptions.startedAt);
        }
        if ('stoppedAt' in transcriptions) {
          transcriptions.stoppedAt = this.encodeDate(transcriptions.stoppedAt);
        }
        if ('updatedAt' in transcriptions) {
          transcriptions.updatedAt = this.encodeDate(transcriptions.updatedAt);
        }
      });
      return data;
    } else {
      return [];
    }
  }
  static transformTranscripts(data) {
    data.forEach(encodedTranscript => {
      if ('peerId' in encodedTranscript) {
        encodedTranscript.peer = HMSEncoder.encodeHmsPeer({
          peerID: encodedTranscript.peerId
        });
      }
    });
  }
}
exports.HMSEncoder = HMSEncoder;
_defineProperty(HMSEncoder, "data", {
  roles: {}
});
//# sourceMappingURL=HMSEncoder.js.map