"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.HMSException = void 0;
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == typeof i ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != typeof t || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != typeof i) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
/**
 * Represents an exception specific to the HMS SDK in a React Native context.
 *
 * This class encapsulates detailed information about errors that can occur while interacting with the HMS SDK.
 * It includes a numeric code to identify the type of error, a human-readable description, and optionally,
 * a message, name, and action that are primarily available on Android. Additionally, it indicates whether
 * the error is terminal (i.e., cannot be recovered from) and whether a retry might be possible.
 *
 * @class HMSException
 * @property {number} code - Numeric error code representing the type of error.
 * @property {string} description - A brief, human-readable description of the error.
 * @property {string} [message] - Additional message information about the error (Android only).
 * @property {string} [name] - The name of the error (Android only).
 * @property {string} [action] - The action during which the error occurred (Android only).
 * @property {boolean} isTerminal - Indicates whether the error is terminal.
 * @property {boolean} [canRetry] - Indicates whether the operation that caused the error can be retried (Android only).
 *
 */
class HMSException {
  // `canRetry` is available only on Android

  constructor(params) {
    _defineProperty(this, "code", void 0);
    _defineProperty(this, "description", void 0);
    _defineProperty(this, "message", void 0);
    // `message` is available only on Android
    _defineProperty(this, "name", void 0);
    // `name` is available only on Android
    _defineProperty(this, "action", void 0);
    // `action` is available only on Android
    _defineProperty(this, "isTerminal", void 0);
    _defineProperty(this, "canRetry", void 0);
    this.code = params.code;
    this.description = params.description;
    this.message = params.message;
    this.name = params.name;
    this.action = params.action;
    this.isTerminal = params.isTerminal;
    this.canRetry = params.canRetry;
  }
}
exports.HMSException = HMSException;
//# sourceMappingURL=HMSException.js.map