"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.HMSInteractivityCenter = void 0;
var _utils = require("../utils");
var _HMSManagerModule = _interopRequireDefault(require("../modules/HMSManagerModule"));
var _HMSConstants = require("./HMSConstants");
var _HMSLogger = require("./HMSLogger");
var _HMSPollsListenerActions = require("./HMSPollsListenerActions");
var _HMSNativeEventListener = _interopRequireDefault(require("./HMSNativeEventListener"));
var _HMSInteractivityEncoder = require("./HMSInteractivityEncoder");
var _HMSHelper = require("./HMSHelper");
var _HMSWhiteboardListenerActions = require("./whiteboard/HMSWhiteboardListenerActions");
var _registerCallbacks = require("./whiteboard/registerCallbacks");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == typeof i ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != typeof t || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != typeof i) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
let pollUpdateSubscription = null;
function registerPollUpdateListener(listener) {
  if (pollUpdateSubscription !== null) {
    return;
  }
  pollUpdateSubscription = _HMSNativeEventListener.default.addListener(_HMSConstants.HMSConstants.DEFAULT_SDK_ID, _HMSPollsListenerActions.HMSPollsListenerActions.ON_POLL_UPDATE, listener);
}
function unregisterPollUpdateListener() {
  if (pollUpdateSubscription === null) {
    return;
  }
  pollUpdateSubscription.remove();
  pollUpdateSubscription = null;
}
class HMSInteractivityCenter {
  onUpdateListenerRemoved(eventType) {
    if (eventType === _HMSPollsListenerActions.HMSPollsListenerActions.ON_POLL_UPDATE) {
      unregisterPollUpdateListener();
    } else if (eventType === _HMSWhiteboardListenerActions.HMSWhiteboardListenerActions.ON_WHITEBOARD_UPDATE) {
      (0, _registerCallbacks.unregisterWhiteboardUpdateListener)();
    } else {
      console.warn('Unknown update listener removed: ', eventType);
    }
  }
  constructor() {
    _defineProperty(this, "_eventEmitter", new _utils.EventEmitter());
    //#region Poll Methods & Listeners
    _defineProperty(this, "_pollUpdateListener", (...args) => {
      const {
        updatedPoll,
        update
      } = args[0];
      _HMSLogger.logger === null || _HMSLogger.logger === void 0 || _HMSLogger.logger.verbose('#Listener ON_POLL_UPDATE', {
        update,
        updatedPoll
      });
      this._eventEmitter.emit(_HMSPollsListenerActions.HMSPollsListenerActions.ON_POLL_UPDATE, _HMSInteractivityEncoder.HMSInteractivityEncoder.transformPoll(updatedPoll), update);
    });
    //#endregion Poll Methods & Listeners
    //#region Whiteboard Methods & Listeners
    _defineProperty(this, "_whiteboardUpdateListener", (...args) => {
      const {
        hmsWhiteboard,
        updateType
      } = args[0];
      _HMSLogger.logger === null || _HMSLogger.logger === void 0 || _HMSLogger.logger.verbose('#Listener ON_WHITEBOARD_UPDATE', {
        updateType,
        hmsWhiteboard
      });
      this._eventEmitter.emit(_HMSWhiteboardListenerActions.HMSWhiteboardListenerActions.ON_WHITEBOARD_UPDATE, _HMSInteractivityEncoder.HMSInteractivityEncoder.transformHMSWhiteboard(hmsWhiteboard), updateType);
    });
    this._eventEmitter.registerOnAllListenersRemoved(this.onUpdateListenerRemoved);
  }
  /**
   * Adds a listener for poll updates
   * @param listener - Callback to be called when a poll is updated
   * @returns HMSEventSubscription
   */
  addPollUpdateListener(listener) {
    registerPollUpdateListener(this._pollUpdateListener);
    return this._eventEmitter.addListener(_HMSPollsListenerActions.HMSPollsListenerActions.ON_POLL_UPDATE, listener, null);
  }

  /**
   * Starts a poll
   *
   * @param pollParams
   */
  async startPoll(pollParams) {
    const transformedPollParams = {
      ...pollParams,
      rolesThatCanVote: pollParams.rolesThatCanVote ? _HMSHelper.HMSHelper.getRoleNames(pollParams.rolesThatCanVote) : undefined,
      rolesThatCanViewResponses: pollParams.rolesThatCanViewResponses ? _HMSHelper.HMSHelper.getRoleNames(pollParams.rolesThatCanViewResponses) : undefined
    };
    const data = {
      ...transformedPollParams,
      id: _HMSConstants.HMSConstants.DEFAULT_SDK_ID
    };
    _HMSLogger.logger === null || _HMSLogger.logger === void 0 || _HMSLogger.logger.verbose('#Function startPoll', JSON.stringify(data));
    return _HMSManagerModule.default.quickStartPoll(data);
  }
  async add(pollQuestionResponseParams) {
    const data = {
      ...pollQuestionResponseParams,
      id: _HMSConstants.HMSConstants.DEFAULT_SDK_ID
    };
    _HMSLogger.logger === null || _HMSLogger.logger === void 0 || _HMSLogger.logger.verbose('#Function add', JSON.stringify(data));
    return _HMSManagerModule.default.addResponseOnPollQuestion(data);
  }
  async stop(pollId) {
    const data = {
      id: _HMSConstants.HMSConstants.DEFAULT_SDK_ID,
      pollId
    };
    _HMSLogger.logger === null || _HMSLogger.logger === void 0 || _HMSLogger.logger.verbose('#Function stop', JSON.stringify(data));
    return _HMSManagerModule.default.stopPoll(data);
  }

  /**
   * Fetches the leaderboard for a poll
   * @param pollId - The id of the poll
   * @param count - The number of entries to fetch
   * @param startIndex - The index to start fetching from
   * @param includeCurrentPeer - Whether to include the current peer in the fetched leaderboard entries
   * @returns Promise<PollLeaderboardResponse>
   */
  async fetchLeaderboard(pollId, count, startIndex, includeCurrentPeer) {
    const data = {
      id: _HMSConstants.HMSConstants.DEFAULT_SDK_ID,
      pollId,
      count,
      startIndex,
      includeCurrentPeer
    };
    _HMSLogger.logger === null || _HMSLogger.logger === void 0 || _HMSLogger.logger.verbose('#Function fetchLeaderboard', data);
    const response = await _HMSManagerModule.default.fetchLeaderboard(data);
    return _HMSInteractivityEncoder.HMSInteractivityEncoder.transformPollLeaderboardResponse(response);
  }
  /**
   * Adds a listener for Whiteboard updates
   * @param listener - Callback to be called when whiteboard is updated
   * @returns HMSEventSubscription
   */
  addWhiteboardUpdateListener(listener) {
    (0, _registerCallbacks.registerWhiteboardUpdateListener)(this._whiteboardUpdateListener);
    return this._eventEmitter.addListener(_HMSWhiteboardListenerActions.HMSWhiteboardListenerActions.ON_WHITEBOARD_UPDATE, listener, null);
  }

  /**
   * Starts Whiteboard
   */
  async startWhiteboard(title) {
    const data = {
      id: _HMSConstants.HMSConstants.DEFAULT_SDK_ID,
      title
    };
    _HMSLogger.logger === null || _HMSLogger.logger === void 0 || _HMSLogger.logger.verbose('#Function startWhiteboard', data);
    return _HMSManagerModule.default.startWhiteboard(data);
  }

  /**
   * Stops Whiteboard
   */
  async stopWhiteboard() {
    const data = {
      id: _HMSConstants.HMSConstants.DEFAULT_SDK_ID
    };
    _HMSLogger.logger === null || _HMSLogger.logger === void 0 || _HMSLogger.logger.verbose('#Function stopWhiteboard', data);
    return _HMSManagerModule.default.stopWhiteboard(data);
  }
  //#endregion Whiteboard Methods & Listeners
}
exports.HMSInteractivityCenter = HMSInteractivityCenter;
//# sourceMappingURL=HMSInteractivityCenter.js.map