"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.HMSInteractivityEncoder = void 0;
var _HMSEncoder = require("./HMSEncoder");
class HMSInteractivityEncoder {
  static transformPoll(poll) {
    if (poll.startedAt) {
      const dateNum = parseInt(poll.startedAt);
      if (isNaN(dateNum)) {
        poll.startedAt = undefined;
      } else {
        poll.startedAt = new Date(dateNum);
      }
    }
    if (poll.stoppedAt) {
      const dateNum = parseInt(poll.stoppedAt);
      if (isNaN(dateNum)) {
        poll.stoppedAt = undefined;
      } else {
        poll.stoppedAt = new Date(dateNum);
      }
    }
    if (poll.createdBy) {
      poll.createdBy = _HMSEncoder.HMSEncoder.encodeHmsPeer(poll.createdBy);
    }
    if (poll.startedBy) {
      poll.startedBy = _HMSEncoder.HMSEncoder.encodeHmsPeer(poll.startedBy);
    }
    if (poll.stoppedBy) {
      poll.stoppedBy = _HMSEncoder.HMSEncoder.encodeHmsPeer(poll.stoppedBy);
    }
    if (poll.rolesThatCanVote) {
      poll.rolesThatCanVote = poll.rolesThatCanVote.map(role => _HMSEncoder.HMSEncoder.encodeHmsRole(role));
    }
    if (poll.rolesThatCanViewResponses) {
      poll.rolesThatCanViewResponses = poll.rolesThatCanViewResponses.map(role => _HMSEncoder.HMSEncoder.encodeHmsRole(role));
    }

    // --- poll.questions: HMSPollQuestion[]
    // --- poll.result: HMSPollResult
    return poll;
  }
  static transformPollLeaderboardResponse(decodedPollLeaderboardResponse) {
    const summary = decodedPollLeaderboardResponse.summary;
    if (summary) {
      if (typeof summary.averageTime === 'string') {
        summary.averageTime = parseInt(summary.averageTime);
      }
    }
    const entries = decodedPollLeaderboardResponse.entries;
    if (entries) {
      entries.forEach(entry => {
        if (typeof entry.duration === 'string') {
          entry.duration = parseInt(entry.duration);
        }
        if (typeof entry.totalResponses === 'string') {
          entry.totalResponses = parseInt(entry.totalResponses);
        }
        if (typeof entry.correctResponses === 'string') {
          entry.correctResponses = parseInt(entry.correctResponses);
        }
        if (typeof entry.position === 'string') {
          entry.position = parseInt(entry.position);
        }
        if (typeof entry.score === 'string') {
          entry.score = parseInt(entry.score);
        }
      });
    }
    return decodedPollLeaderboardResponse;
  }
  static transformHMSWhiteboard(hmsWhiteboard) {
    if (hmsWhiteboard.owner) {
      hmsWhiteboard.owner = _HMSEncoder.HMSEncoder.encodeHmsPeer(hmsWhiteboard.owner);
    }
    return hmsWhiteboard;
  }
}
exports.HMSInteractivityEncoder = HMSInteractivityEncoder;
//# sourceMappingURL=HMSInteractivityEncoder.js.map