"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.HMSLogSettings = void 0;
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == typeof i ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != typeof t || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != typeof i) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
/**
 * Represents the logging settings for the HMS SDK.
 *
 * This class encapsulates the settings related to logging within the HMS SDK, including the log level,
 * whether log storage is enabled, and the maximum directory size for log storage.
 * It is used to configure how logs are handled by the SDK, allowing for customization of log verbosity and storage requirements.
 *
 * @class HMSLogSettings
 * @property {HMSLogLevel} level - The log level setting, determining the verbosity of the logs.
 * @property {boolean} isLogStorageEnabled - Flag indicating whether log storage is enabled. Android only.
 * @property {HMSLogAlarmManager} maxDirSizeInBytes - The maximum size of the log directory in bytes, managed by an instance of HMSLogAlarmManager. Android only.
 *
 * @see https://www.100ms.live/docs/react-native/v2/how-to-guides/debugging/logger
 */
class HMSLogSettings {
  constructor(params) {
    _defineProperty(this, "level", void 0);
    _defineProperty(this, "isLogStorageEnabled", void 0);
    _defineProperty(this, "maxDirSizeInBytes", void 0);
    this.level = params.level;
    this.isLogStorageEnabled = params.isLogStorageEnabled;
    this.maxDirSizeInBytes = params.maxDirSizeInBytes;
  }
}
exports.HMSLogSettings = HMSLogSettings;
//# sourceMappingURL=HMSLogSettings.js.map