"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setLogger = exports.logger = exports.getLogger = exports.HMSLogger = void 0;
var _HMSLogLevel = require("./HMSLogLevel");
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == typeof i ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != typeof t || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != typeof i) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
let logger = exports.logger = void 0;
const getLogger = () => {
  return logger;
};
exports.getLogger = getLogger;
const setLogger = (id, hmsLogger) => {
  exports.logger = logger = hmsLogger;
  hmsLogger === null || hmsLogger === void 0 || hmsLogger.verbose('#Function setLogger', {
    id
  });
};
exports.setLogger = setLogger;
class HMSLogger {
  constructor(params) {
    _defineProperty(this, "_verbose", false);
    _defineProperty(this, "_warning", false);
    _defineProperty(this, "_error", false);
    _defineProperty(this, "onLog", void 0);
    if (params) {
      this._verbose = params.verbose;
      this._warning = params.warning;
      this._error = params.error;
    }
  }
  verbose(message, data) {
    if (this._verbose) {
      const timestamp = this.getTimestamp();
      console.log(timestamp, message, data);
      if (this.onLog) {
        this === null || this === void 0 || this.onLog({
          timestamp,
          message,
          data
        });
      }
    }
  }
  warn(message, data) {
    if (this._warning) {
      const timestamp = this.getTimestamp();
      console.warn(timestamp, message, data);
      if (this.onLog) {
        this === null || this === void 0 || this.onLog({
          timestamp,
          message,
          data
        });
      }
    }
  }
  error(message, data) {
    if (this._error) {
      const timestamp = this.getTimestamp();
      console.error(timestamp, message, data);
      if (this.onLog) {
        this === null || this === void 0 || this.onLog({
          timestamp,
          message,
          data
        });
      }
    }
  }
  setLogListener(callback) {
    this.onLog = callback;
  }
  updateLogLevel(level, value) {
    switch (level) {
      case _HMSLogLevel.HMSLogLevel.VERBOSE:
        {
          this._verbose = value;
          return;
        }
      case _HMSLogLevel.HMSLogLevel.WARNING:
        {
          this._warning = value;
          return;
        }
      case _HMSLogLevel.HMSLogLevel.ERROR:
        {
          this._error = value;
          return;
        }
      default:
        {
          return;
        }
    }
  }
  getTimestamp() {
    const d = new Date();
    const hh = d.getHours().toString().padStart(2, '0');
    const mm = d.getMinutes().toString().padStart(2, '0');
    const ss = d.getSeconds().toString().padStart(2, '0');
    const millis = d.getMilliseconds().toString().padStart(3, '0');
    return `${hh}:${mm}:${ss}:${millis}`;
  }
}
exports.HMSLogger = HMSLogger;
//# sourceMappingURL=HMSLogger.js.map