"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.HMSNativeEventEmitter = void 0;
var _reactNative = require("react-native");
var _HMSLogger = require("./HMSLogger");
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == typeof i ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != typeof t || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != typeof i) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
class HMSNativeEventEmitter {
  constructor(nativeModule) {
    _defineProperty(this, "_nativeEventEmitter", void 0);
    _defineProperty(this, "_nativeModule", void 0);
    this._nativeModule = nativeModule;
    this._nativeEventEmitter = new _reactNative.NativeEventEmitter(nativeModule);
  }
  listenerCount(eventType) {
    // For RN version < v0.64, listenerCount method is not available
    // @ts-ignore - typescript is giving error because we are running it with RN types version >= 0.64
    if (typeof this._nativeEventEmitter.listeners === 'function') {
      // @ts-ignore - typescript is giving error because we are running it with RN types version >= 0.64
      const count = this._nativeEventEmitter.listeners(eventType).length;
      _HMSLogger.logger === null || _HMSLogger.logger === void 0 || _HMSLogger.logger.verbose('#Function HMSNativeEventEmitter.listenerCount', {
        eventType,
        count
      });
      return count;
    }

    // For RN version >= v0.64
    const count = this._nativeEventEmitter.listenerCount(eventType);
    _HMSLogger.logger === null || _HMSLogger.logger === void 0 || _HMSLogger.logger.verbose('#Function HMSNativeEventEmitter.listenerCount', {
      eventType,
      count
    });
    return count;
  }
  addListener(id, eventType, listener) {
    _HMSLogger.logger === null || _HMSLogger.logger === void 0 || _HMSLogger.logger.verbose('#Function HMSNativeEventEmitter.addListener', {
      id,
      eventType,
      listener
    });

    // enable `eventType` on `HMSSDK`, if no listeners were added before
    // if some listeners were added before, then `eventType` on `HMSSDK` should already be enabled
    if (this.listenerCount(eventType) <= 0) {
      this.enableHMSEventType(id, eventType);
    }
    let subscription = this._nativeEventEmitter.addListener(eventType, listener);
    return {
      remove: () => {
        if (subscription && Object.getOwnPropertyNames(subscription).includes('remove') && typeof subscription.remove === 'function') {
          subscription.remove();
          subscription = null;
        } else {
          this._nativeEventEmitter.removeListener(eventType, listener);
        }

        // disable `eventType` on `HMSSDK`, if all listeners has been removed
        if (this.listenerCount(eventType) <= 0) {
          this.disableHMSEventType(id, eventType);
        }
      }
    };
  }
  removeAllListeners(id, eventType) {
    _HMSLogger.logger === null || _HMSLogger.logger === void 0 || _HMSLogger.logger.verbose('#Function HMSNativeEventEmitter.removeAllListeners', {
      id,
      eventType
    });

    // disable `eventType` on `HMSSDK`
    this.disableHMSEventType(id, eventType);
    return this._nativeEventEmitter.removeAllListeners(eventType);
  }
  enableHMSEventType(id, eventType) {
    _HMSLogger.logger === null || _HMSLogger.logger === void 0 || _HMSLogger.logger.verbose('#Function HMSNativeEventEmitter.enableHMSEventType', {
      id,
      eventType
    });
    this._nativeModule.enableEvent({
      id,
      eventType
    });
  }
  disableHMSEventType(id, eventType) {
    _HMSLogger.logger === null || _HMSLogger.logger === void 0 || _HMSLogger.logger.verbose('#Function HMSNativeEventEmitter.disableHMSEventType', {
      id,
      eventType
    });
    this._nativeModule.disableEvent({
      id,
      eventType
    });
  }
}
exports.HMSNativeEventEmitter = HMSNativeEventEmitter;
//# sourceMappingURL=HMSNativeEventEmitter.js.map