"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.HMSRemoteVideoTrack = void 0;
var _reactNative = require("react-native");
var _HMSLogger = require("./HMSLogger");
var _HMSVideoTrack = require("./HMSVideoTrack");
var _HMSEncoder = require("./HMSEncoder");
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == typeof i ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != typeof t || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != typeof i) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
const {
  /**
   * @ignore
   */
  HMSManager
} = _reactNative.NativeModules;
class HMSRemoteVideoTrack extends _HMSVideoTrack.HMSVideoTrack {
  /**
   * Switches Video of remote user on/off depending upon the value of playbackAllowed
   *
   * @param {boolean} playbackAllowed
   * @memberof HMSRemoteVideoTrack
   */
  setPlaybackAllowed(playbackAllowed) {
    const logger = (0, _HMSLogger.getLogger)();
    logger === null || logger === void 0 || logger.verbose('#Function setPlaybackAllowed', {
      trackId: this.trackId,
      id: this.id,
      source: this.source,
      type: this.type,
      playbackAllowed
    });
    HMSManager.setPlaybackAllowed({
      id: this.id,
      trackId: this.trackId,
      playbackAllowed
    });
  }
  async getLayer() {
    const logger = (0, _HMSLogger.getLogger)();
    logger === null || logger === void 0 || logger.verbose('#Function getLayer', {
      id: this.id,
      trackId: this.trackId
    });
    const layer = await HMSManager.getVideoTrackLayer({
      id: this.id,
      trackId: this.trackId
    });
    return layer;
  }
  async getLayerDefinition() {
    const logger = (0, _HMSLogger.getLogger)();
    logger === null || logger === void 0 || logger.verbose('#Function getLayerDefinition', {
      id: this.id,
      trackId: this.trackId
    });
    const layerDefinition = await HMSManager.getVideoTrackLayerDefinition({
      id: this.id,
      trackId: this.trackId
    });
    return _HMSEncoder.HMSEncoder.encodeHMSSimulcastLayerDefinition(layerDefinition);
  }
  async setLayer(layer) {
    const logger = (0, _HMSLogger.getLogger)();
    logger === null || logger === void 0 || logger.verbose('#Function setVideoTrackLayer', {
      id: this.id,
      trackId: this.trackId,
      layer
    });
    const success = await HMSManager.setVideoTrackLayer({
      id: this.id,
      trackId: this.trackId,
      layer
    });
    return success;
  }
  constructor(params) {
    super(params);
    _defineProperty(this, "isPlaybackAllowed", async () => {
      try {
        const logger = (0, _HMSLogger.getLogger)();
        logger === null || logger === void 0 || logger.verbose('#Function isPlaybackAllowed', {
          trackId: this.trackId,
          id: this.id,
          source: this.source,
          type: this.type
        });
        const val = await HMSManager.isPlaybackAllowed({
          id: this.id,
          trackId: this.trackId
        });
        return val;
      } catch (e) {
        return true;
      }
    });
  }
}
exports.HMSRemoteVideoTrack = HMSRemoteVideoTrack;
//# sourceMappingURL=HMSRemoteVideoTrack.js.map