"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.HMSRole = void 0;
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == typeof i ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != typeof t || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != typeof i) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
/**
 * Represents a role within the HMS ecosystem.
 *
 * A role defines the capabilities and permissions a user has in a session, such as publishing or subscribing to streams,
 * and the priority of the user in the session. This class encapsulates the properties of a role, including its name,
 * publishing settings, subscribing settings, permissions, and priority.
 *
 * @class HMSRole
 * @property {string} [name] - The name of the role.
 * @property {HMSPublishSettings} [publishSettings] - Settings related to publishing streams for this role.
 * @property {HMSSubscribeSettings} [subscribeSettings] - Settings related to subscribing to streams for this role.
 * @property {HMSPermissions} [permissions] - Permissions granted to this role.
 * @property {number} [priority] - The priority of the role, determining its precedence in the session.
 *
 * @see https://www.100ms.live/docs/get-started/v2/get-started/concepts/templates-and-roles
 *
 */
class HMSRole {
  constructor(params) {
    _defineProperty(this, "name", void 0);
    _defineProperty(this, "publishSettings", void 0);
    _defineProperty(this, "subscribeSettings", void 0);
    _defineProperty(this, "permissions", void 0);
    _defineProperty(this, "priority", void 0);
    if (params) {
      this.name = params.name;
      this.priority = params.priority;
      this.permissions = params.permissions;
      this.publishSettings = params.publishSettings;
      this.subscribeSettings = params.subscribeSettings;
    }
  }
}
exports.HMSRole = HMSRole;
//# sourceMappingURL=HMSRole.js.map