"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.clearHmsRoomCache = exports.HMSRoomCache = void 0;
exports.getHMSRoomPropertyFromNative = getHMSRoomPropertyFromNative;
exports.setHmsRoomCache = exports.hmsRoomCache = exports.getHmsRoomCache = void 0;
var _HMSManagerModule = _interopRequireDefault(require("../modules/HMSManagerModule"));
var _HMSEncoder = require("./HMSEncoder");
var _HMSRoomUpdate = require("./HMSRoomUpdate");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == typeof i ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != typeof t || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != typeof i) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
let hmsRoomCache = exports.hmsRoomCache = void 0;
const getHmsRoomCache = () => hmsRoomCache;
exports.getHmsRoomCache = getHmsRoomCache;
const setHmsRoomCache = _hmsRoomCache => {
  exports.hmsRoomCache = hmsRoomCache = _hmsRoomCache;
};
exports.setHmsRoomCache = setHmsRoomCache;
const clearHmsRoomCache = () => {
  if (hmsRoomCache) {
    hmsRoomCache.cleanup();
  }
  exports.hmsRoomCache = hmsRoomCache = undefined;
};
exports.clearHmsRoomCache = clearHmsRoomCache;
class HMSRoomCache {
  constructor(id) {
    _defineProperty(this, "_data", new Map());
    _defineProperty(this, "id", void 0);
    this.id = id;
  }
  getProperty(property) {
    const value = this._data.get(property);
    if (value) {
      return value;
    }
    const encodedValue = getHMSRoomPropertyFromNative(this.id, property);
    this._data.set(property, encodedValue);
    return encodedValue;
  }
  updateRoomCache(data, updateType) {
    switch (updateType) {
      case _HMSRoomUpdate.HMSRoomUpdate.ROOM_MUTED:
      case _HMSRoomUpdate.HMSRoomUpdate.ROOM_UNMUTED:
        {
          break;
        }
      case _HMSRoomUpdate.HMSRoomUpdate.SERVER_RECORDING_STATE_UPDATED:
        {
          this._data.set('serverRecordingState', _HMSEncoder.HMSEncoder.encodeServerRecordingState(data.serverRecordingState));
          break;
        }
      case _HMSRoomUpdate.HMSRoomUpdate.RTMP_STREAMING_STATE_UPDATED:
        {
          this._data.set('rtmpHMSRtmpStreamingState', _HMSEncoder.HMSEncoder.encodeRTMPStreamingState(data.rtmpHMSRtmpStreamingState));
          break;
        }
      case _HMSRoomUpdate.HMSRoomUpdate.BROWSER_RECORDING_STATE_UPDATED:
        {
          this._data.set('browserRecordingState', _HMSEncoder.HMSEncoder.encodeBrowserRecordingState(data.browserRecordingState));
          break;
        }
      case _HMSRoomUpdate.HMSRoomUpdate.HLS_STREAMING_STATE_UPDATED:
        {
          this._data.set('hlsStreamingState', _HMSEncoder.HMSEncoder.encodeHLSStreamingState(data.hlsStreamingState));
          break;
        }
      case _HMSRoomUpdate.HMSRoomUpdate.HLS_RECORDING_STATE_UPDATED:
        {
          this._data.set('hlsRecordingState', _HMSEncoder.HMSEncoder.encodeHLSRecordingState(data.hlsRecordingState));
          break;
        }
      case _HMSRoomUpdate.HMSRoomUpdate.ROOM_PEER_COUNT_UPDATED:
        {
          this._data.set('peerCount', encodeHMSRoomProperty(this.id, 'peerCount', data));
          break;
        }
      case _HMSRoomUpdate.HMSRoomUpdate.TRANSCRIPTIONS_UPDATED:
        {
          this._data.set('transcriptions', encodeHMSRoomProperty(this.id, 'transcriptions', data));
          break;
        }
      default:
        {
          for (const key in data) {
            const property = key;
            if (Object.prototype.hasOwnProperty.call(data, property)) {
              const value = encodeHMSRoomProperty(this.id, property, data);
              this._data.set(property, value);
            }
          }
          break;
        }
    }
  }
  cleanup() {
    this._data.clear();
  }
}
exports.HMSRoomCache = HMSRoomCache;
function getHMSRoomPropertyFromNative(id, property) {
  const data = _HMSManagerModule.default.getRoomProperty({
    id,
    property
  });
  if (!data) return data;
  const value = encodeHMSRoomProperty(id, property, data);
  return value;
}
function encodeHMSRoomProperty(id, property, data) {
  let value;
  if (property === 'peers') {
    value = _HMSEncoder.HMSEncoder.encodeHmsPeers(data.peers);
  } else if (property === 'browserRecordingState') {
    value = _HMSEncoder.HMSEncoder.encodeBrowserRecordingState(data.browserRecordingState);
  } else if (property === 'rtmpHMSRtmpStreamingState') {
    value = _HMSEncoder.HMSEncoder.encodeRTMPStreamingState(data.rtmpHMSRtmpStreamingState);
  } else if (property === 'serverRecordingState') {
    value = _HMSEncoder.HMSEncoder.encodeServerRecordingState(data.serverRecordingState);
  } else if (property === 'hlsStreamingState') {
    value = _HMSEncoder.HMSEncoder.encodeHLSStreamingState(data.hlsStreamingState);
  } else if (property === 'hlsRecordingState') {
    value = _HMSEncoder.HMSEncoder.encodeHLSRecordingState(data.hlsRecordingState);
  } else if (property === 'transcriptions') {
    value = _HMSEncoder.HMSEncoder.encodeTranscriptions(data.transcriptions);
  } else if (property === 'localPeer') {
    value = _HMSEncoder.HMSEncoder.encodeHmsLocalPeer(data.localPeer, id);
  } else if (property === 'peerCount') {
    value = typeof data[property] === 'number' ? data[property] : null;
  } else {
    value = data[property];
  }
  return value;
}
//# sourceMappingURL=HMSRoomCache.js.map