"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useIsHLSStreamLive = exports.useHMSHLSPlayerSubtitles = exports.useHMSHLSPlayerStatsError = exports.useHMSHLSPlayerStats = exports.useHMSHLSPlayerStat = exports.useHMSHLSPlayerResolution = exports.useHMSHLSPlayerPlaybackState = exports.useHMSHLSPlayerPlaybackError = exports.useHMSHLSPlayerCue = exports.setHMSHLSPlayerSubtitles = exports.setHMSHLSPlayerStatsError = exports.setHMSHLSPlayerStats = exports.setHMSHLSPlayerResolution = exports.setHMSHLSPlayerPlaybackState = exports.setHMSHLSPlayerPlaybackError = exports.setHMSHLSPlayerCue = void 0;
var _react = require("react");
var _reactNative = require("react-native");
var _shallow = require("zustand/shallow");
var _hlsPlayerStatsStore = require("../../stores/hls-player-stats-store");
var _hmsStore = require("../../stores/hms-store");
// use latest state (with component rerender)

const useHMSHLSPlayerPlaybackState = () => {
  return (0, _hmsStore.useHMSStore)(state => state.playbackState);
};
exports.useHMSHLSPlayerPlaybackState = useHMSHLSPlayerPlaybackState;
const useHMSHLSPlayerResolution = () => {
  return (0, _hmsStore.useHMSStore)(state => state.resolution);
};
exports.useHMSHLSPlayerResolution = useHMSHLSPlayerResolution;
const useHMSHLSPlayerStats = () => {
  return (0, _hlsPlayerStatsStore.useHMSHLSPlayerStatsStore)(state => ({
    stats: state.stats,
    error: state.error
  }), _shallow.shallow);
};
exports.useHMSHLSPlayerStats = useHMSHLSPlayerStats;
const useHMSHLSPlayerStat = stat => {
  return (0, _hlsPlayerStatsStore.useHMSHLSPlayerStatsStore)(state => state.stats[stat]);
};
exports.useHMSHLSPlayerStat = useHMSHLSPlayerStat;
const useHMSHLSPlayerSubtitles = () => {
  return (0, _hlsPlayerStatsStore.useHMSHLSPlayerStatsStore)(state => state.subtitles);
};

// The distance of current playback position from the live edge of HLS stream
exports.useHMSHLSPlayerSubtitles = useHMSHLSPlayerSubtitles;
const useIsHLSStreamLive = (liveOffsetMillis = _reactNative.Platform.select({
  default: 10000,
  ios: 5000
})) => {
  return (0, _hlsPlayerStatsStore.useHMSHLSPlayerStatsStore)(state => state.stats.distanceFromLive < liveOffsetMillis);
};

// get latest state (without component rerender)
exports.useIsHLSStreamLive = useIsHLSStreamLive;
const useHMSHLSPlayerStatsError = (callback, deps) => {
  (0, _react.useEffect)(() => {
    return _hlsPlayerStatsStore.useHMSHLSPlayerStatsStore.subscribe(state => state.error, data => {
      if (data) callback(data);
    });
  }, deps);
};
exports.useHMSHLSPlayerStatsError = useHMSHLSPlayerStatsError;
const useHMSHLSPlayerPlaybackError = (callback, deps) => {
  (0, _react.useEffect)(() => {
    return _hmsStore.useHMSStore.subscribe(state => state.error, data => {
      if (data) callback(data);
    });
  }, deps);
};
exports.useHMSHLSPlayerPlaybackError = useHMSHLSPlayerPlaybackError;
const useHMSHLSPlayerCue = (callback, deps) => {
  (0, _react.useEffect)(() => {
    return _hmsStore.useHMSStore.subscribe(state => state.cue, data => {
      if (data) callback(data);
    });
  }, deps);
};

// state setters
exports.useHMSHLSPlayerCue = useHMSHLSPlayerCue;
const setHMSHLSPlayerPlaybackState = exports.setHMSHLSPlayerPlaybackState = _hmsStore.useHMSStore.getState().setPlaybackState;
const setHMSHLSPlayerResolution = exports.setHMSHLSPlayerResolution = _hmsStore.useHMSStore.getState().setResolution;
const setHMSHLSPlayerCue = exports.setHMSHLSPlayerCue = _hmsStore.useHMSStore.getState().setCue;
const setHMSHLSPlayerPlaybackError = exports.setHMSHLSPlayerPlaybackError = _hmsStore.useHMSStore.getState().setPlaybackError;
const setHMSHLSPlayerStats = exports.setHMSHLSPlayerStats = _hlsPlayerStatsStore.useHMSHLSPlayerStatsStore.getState().changeStats;
const setHMSHLSPlayerStatsError = exports.setHMSHLSPlayerStatsError = _hlsPlayerStatsStore.useHMSHLSPlayerStatsStore.getState().setError;
const setHMSHLSPlayerSubtitles = exports.setHMSHLSPlayerSubtitles = _hlsPlayerStatsStore.useHMSHLSPlayerStatsStore.getState().setSubtitles;
//# sourceMappingURL=hooks.js.map