"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useHMSPeerUpdates = void 0;
var React = _interopRequireWildcard(require("react"));
var _HMSConstants = require("../classes/HMSConstants");
var _HMSEncoder = require("../classes/HMSEncoder");
var _HMSEventEmitter = require("../classes/HMSEventEmitter");
var _HMSLogger = require("../classes/HMSLogger");
var _HMSNativeEventListener = _interopRequireDefault(require("../classes/HMSNativeEventListener"));
var _HMSPeerUpdate = require("../classes/HMSPeerUpdate");
var _HMSPeersCache = require("../classes/HMSPeersCache");
var _HMSUpdateListenerActions = require("../classes/HMSUpdateListenerActions");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function (e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
const eventType = _HMSUpdateListenerActions.HMSUpdateListenerActions.ON_PEER_UPDATE;

// TODO: How to handle events which have already been received?
// TODO(DOUBT): add HMSInstance as dependency because of actions enabling/disabling events on HMSInstance?
const useHMSPeerUpdates = (effect, deps = []) => {
  React.useEffect(() => {
    if (_HMSEventEmitter.HMSEventEmitter.listenerCount(eventType) <= 0) {
      addPeerUpdateSubscription();
    }
    const subscription = _HMSEventEmitter.HMSEventEmitter.addListener(eventType, effect, null);
    return () => {
      subscription.remove();
      if (_HMSEventEmitter.HMSEventEmitter.listenerCount(eventType) <= 0) {
        removePeerUpdateSubscription();
      }
    };
  }, deps);
};
exports.useHMSPeerUpdates = useHMSPeerUpdates;
let peerUpdateSubscription = null;
function addPeerUpdateSubscription() {
  if (peerUpdateSubscription !== null) {
    return;
  }
  peerUpdateSubscription = _HMSNativeEventListener.default.addListener(_HMSConstants.HMSConstants.DEFAULT_SDK_ID, _HMSUpdateListenerActions.HMSUpdateListenerActions.ON_PEER_UPDATE, peerData => {
    var _getHmsPeersCache, _getLogger;
    const data = {
      peer: peerData,
      type: null
    };
    for (const ordinal of _HMSPeerUpdate.HMSPeerUpdateOrdinals.keys()) {
      if (ordinal in peerData) {
        data.peer.peerID = peerData[ordinal];
        data.type = ordinal;
        break;
      }
    }
    const peer = _HMSEncoder.HMSEncoder.encodeHmsPeer(data.peer);
    const type = _HMSEncoder.HMSEncoder.encodeHmsPeerUpdate(data.type) || data.type;
    (_getHmsPeersCache = (0, _HMSPeersCache.getHmsPeersCache)()) === null || _getHmsPeersCache === void 0 || _getHmsPeersCache.updatePeerCache(data.peer.peerID, data.peer, type);
    (_getLogger = (0, _HMSLogger.getLogger)()) === null || _getLogger === void 0 || _getLogger.verbose('#Listener ON_PEER_LISTENER_CALL', {
      peer,
      type
    });
    _HMSEventEmitter.HMSEventEmitter.emit(eventType, {
      peer,
      type
    });
  });
}
function removePeerUpdateSubscription() {
  if (peerUpdateSubscription === null) {
    return;
  }
  peerUpdateSubscription.remove();
  peerUpdateSubscription = null;
}
//# sourceMappingURL=useHMSPeerUpdates.js.map