"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SoftInputModes = void 0;
exports.getSoftInputMode = getSoftInputMode;
exports.setSoftInputMode = setSoftInputMode;
var _reactNative = require("react-native");
var _HMSManagerModule = _interopRequireDefault(require("../modules/HMSManagerModule"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
let SoftInputModes = exports.SoftInputModes = /*#__PURE__*/function (SoftInputModes) {
  SoftInputModes[SoftInputModes["SOFT_INPUT_ADJUST_NOTHING"] = 48] = "SOFT_INPUT_ADJUST_NOTHING";
  SoftInputModes[SoftInputModes["SOFT_INPUT_ADJUST_PAN"] = 32] = "SOFT_INPUT_ADJUST_PAN";
  SoftInputModes[SoftInputModes["SOFT_INPUT_ADJUST_RESIZE"] = 16] = "SOFT_INPUT_ADJUST_RESIZE";
  SoftInputModes[SoftInputModes["SOFT_INPUT_ADJUST_UNSPECIFIED"] = 0] = "SOFT_INPUT_ADJUST_UNSPECIFIED";
  return SoftInputModes;
}({});
/**
 * [Android Only] Sets the `windowSoftInputMode` value for the current activity
 * @param inputMode window softinput moede to use. It can be any variant of the {@link SoftInputModes} enum
 */
function setSoftInputMode(inputMode) {
  if (_reactNative.Platform.OS === 'android') {
    const result = _HMSManagerModule.default.setSoftInputMode(inputMode);
    if (result === -1) {
      throw new Error("Couln't get AppContext, Activity or Window!");
    }
  }
}

/**
 * [Android Only] Gets the current `windowSoftInputMode` value of the current activity
 * @returns current `windowSoftInputMode` ({@link SoftInputModes}) value of the current activity
 */
function getSoftInputMode() {
  if (_reactNative.Platform.OS === 'android') {
    const result = _HMSManagerModule.default.getSoftInputMode();
    if (result === -1) {
      throw new Error("Couln't get AppContext, Activity or Window!");
    }
    return result;
  }
  return SoftInputModes.SOFT_INPUT_ADJUST_UNSPECIFIED;
}
//# sourceMappingURL=keyboard.js.map