function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == typeof i ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != typeof t || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != typeof i) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
import { EventEmitter } from '../utils';
import HMSManager from '../modules/HMSManagerModule';
import { HMSConstants } from './HMSConstants';
import { logger } from './HMSLogger';
import { HMSPollsListenerActions } from './HMSPollsListenerActions';
import HMSNativeEventListener from './HMSNativeEventListener';
import { HMSInteractivityEncoder } from './HMSInteractivityEncoder';
import { HMSHelper } from './HMSHelper';
import { HMSWhiteboardListenerActions } from './whiteboard/HMSWhiteboardListenerActions';
import { registerWhiteboardUpdateListener, unregisterWhiteboardUpdateListener } from './whiteboard/registerCallbacks';
let pollUpdateSubscription = null;
function registerPollUpdateListener(listener) {
  if (pollUpdateSubscription !== null) {
    return;
  }
  pollUpdateSubscription = HMSNativeEventListener.addListener(HMSConstants.DEFAULT_SDK_ID, HMSPollsListenerActions.ON_POLL_UPDATE, listener);
}
function unregisterPollUpdateListener() {
  if (pollUpdateSubscription === null) {
    return;
  }
  pollUpdateSubscription.remove();
  pollUpdateSubscription = null;
}
export class HMSInteractivityCenter {
  onUpdateListenerRemoved(eventType) {
    if (eventType === HMSPollsListenerActions.ON_POLL_UPDATE) {
      unregisterPollUpdateListener();
    } else if (eventType === HMSWhiteboardListenerActions.ON_WHITEBOARD_UPDATE) {
      unregisterWhiteboardUpdateListener();
    } else {
      console.warn('Unknown update listener removed: ', eventType);
    }
  }
  constructor() {
    _defineProperty(this, "_eventEmitter", new EventEmitter());
    //#region Poll Methods & Listeners
    _defineProperty(this, "_pollUpdateListener", (...args) => {
      const {
        updatedPoll,
        update
      } = args[0];
      logger === null || logger === void 0 || logger.verbose('#Listener ON_POLL_UPDATE', {
        update,
        updatedPoll
      });
      this._eventEmitter.emit(HMSPollsListenerActions.ON_POLL_UPDATE, HMSInteractivityEncoder.transformPoll(updatedPoll), update);
    });
    //#endregion Poll Methods & Listeners
    //#region Whiteboard Methods & Listeners
    _defineProperty(this, "_whiteboardUpdateListener", (...args) => {
      const {
        hmsWhiteboard,
        updateType
      } = args[0];
      logger === null || logger === void 0 || logger.verbose('#Listener ON_WHITEBOARD_UPDATE', {
        updateType,
        hmsWhiteboard
      });
      this._eventEmitter.emit(HMSWhiteboardListenerActions.ON_WHITEBOARD_UPDATE, HMSInteractivityEncoder.transformHMSWhiteboard(hmsWhiteboard), updateType);
    });
    this._eventEmitter.registerOnAllListenersRemoved(this.onUpdateListenerRemoved);
  }
  /**
   * Adds a listener for poll updates
   * @param listener - Callback to be called when a poll is updated
   * @returns HMSEventSubscription
   */
  addPollUpdateListener(listener) {
    registerPollUpdateListener(this._pollUpdateListener);
    return this._eventEmitter.addListener(HMSPollsListenerActions.ON_POLL_UPDATE, listener, null);
  }

  /**
   * Starts a poll
   *
   * @param pollParams
   */
  async startPoll(pollParams) {
    const transformedPollParams = {
      ...pollParams,
      rolesThatCanVote: pollParams.rolesThatCanVote ? HMSHelper.getRoleNames(pollParams.rolesThatCanVote) : undefined,
      rolesThatCanViewResponses: pollParams.rolesThatCanViewResponses ? HMSHelper.getRoleNames(pollParams.rolesThatCanViewResponses) : undefined
    };
    const data = {
      ...transformedPollParams,
      id: HMSConstants.DEFAULT_SDK_ID
    };
    logger === null || logger === void 0 || logger.verbose('#Function startPoll', JSON.stringify(data));
    return HMSManager.quickStartPoll(data);
  }
  async add(pollQuestionResponseParams) {
    const data = {
      ...pollQuestionResponseParams,
      id: HMSConstants.DEFAULT_SDK_ID
    };
    logger === null || logger === void 0 || logger.verbose('#Function add', JSON.stringify(data));
    return HMSManager.addResponseOnPollQuestion(data);
  }
  async stop(pollId) {
    const data = {
      id: HMSConstants.DEFAULT_SDK_ID,
      pollId
    };
    logger === null || logger === void 0 || logger.verbose('#Function stop', JSON.stringify(data));
    return HMSManager.stopPoll(data);
  }

  /**
   * Fetches the leaderboard for a poll
   * @param pollId - The id of the poll
   * @param count - The number of entries to fetch
   * @param startIndex - The index to start fetching from
   * @param includeCurrentPeer - Whether to include the current peer in the fetched leaderboard entries
   * @returns Promise<PollLeaderboardResponse>
   */
  async fetchLeaderboard(pollId, count, startIndex, includeCurrentPeer) {
    const data = {
      id: HMSConstants.DEFAULT_SDK_ID,
      pollId,
      count,
      startIndex,
      includeCurrentPeer
    };
    logger === null || logger === void 0 || logger.verbose('#Function fetchLeaderboard', data);
    const response = await HMSManager.fetchLeaderboard(data);
    return HMSInteractivityEncoder.transformPollLeaderboardResponse(response);
  }
  /**
   * Adds a listener for Whiteboard updates
   * @param listener - Callback to be called when whiteboard is updated
   * @returns HMSEventSubscription
   */
  addWhiteboardUpdateListener(listener) {
    registerWhiteboardUpdateListener(this._whiteboardUpdateListener);
    return this._eventEmitter.addListener(HMSWhiteboardListenerActions.ON_WHITEBOARD_UPDATE, listener, null);
  }

  /**
   * Starts Whiteboard
   */
  async startWhiteboard(title) {
    const data = {
      id: HMSConstants.DEFAULT_SDK_ID,
      title
    };
    logger === null || logger === void 0 || logger.verbose('#Function startWhiteboard', data);
    return HMSManager.startWhiteboard(data);
  }

  /**
   * Stops Whiteboard
   */
  async stopWhiteboard() {
    const data = {
      id: HMSConstants.DEFAULT_SDK_ID
    };
    logger === null || logger === void 0 || logger.verbose('#Function stopWhiteboard', data);
    return HMSManager.stopWhiteboard(data);
  }
  //#endregion Whiteboard Methods & Listeners
}
//# sourceMappingURL=HMSInteractivityCenter.js.map