function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == typeof i ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != typeof t || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != typeof i) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
import HMSManager from '../modules/HMSManagerModule';
import { HMSEncoder } from './HMSEncoder';
import { HMSPeerUpdate } from './HMSPeerUpdate';
import { HMSTrackUpdate } from './HMSTrackUpdate';
import { HMSTrackType } from './HMSTrackType';
import { HMSTrackSource } from './HMSTrackSource';
export let hmsPeersCache;
export const getHmsPeersCache = () => {
  return hmsPeersCache;
};
export const setHmsPeersCache = _hmsPeersCache => {
  hmsPeersCache = _hmsPeersCache;
};
export const clearHmsPeersCache = () => {
  if (hmsPeersCache) {
    hmsPeersCache.cleanup();
  }
  hmsPeersCache = undefined;
};
export class HMSPeersCache {
  constructor(id) {
    _defineProperty(this, "_data", new Map());
    _defineProperty(this, "id", void 0);
    this.id = id;
  }
  getProperty(peerId, property) {
    const peerObj = this._data.get(peerId);
    if (peerObj && property in peerObj) {
      return peerObj[property];
    }
    const encodedValue = getPeerPropertyFromNative(this.id, peerId, property);
    if (!peerObj) {
      this._data.set(peerId, {
        [property]: encodedValue
      });
    } else {
      peerObj[property] = encodedValue;
    }
    return encodedValue;
  }
  updatePeerCache(peerId, data, updateType) {
    const peerObj = this._data.get(peerId);
    if (updateType === HMSPeerUpdate.PEER_JOINED) {
      if (!peerObj) {
        this._data.set(peerId, data);
      }
      return;
    }
    if (updateType === HMSPeerUpdate.PEER_LEFT) {
      this._data.delete(peerId);
      return;
    }
    let updatedObj = peerObj || {};
    switch (updateType) {
      case HMSTrackUpdate.TRACK_ADDED:
        {
          const track = data.track;
          if (track.source === HMSTrackSource.REGULAR) {
            if (track.type === HMSTrackType.VIDEO) {
              updatedObj.videoTrack = track;
              updatedObj.videoTrack.isDegraded = false;
            } else if (track.type === HMSTrackType.AUDIO) {
              updatedObj.audioTrack = track;
            }
          } else {
            if (Array.isArray(updatedObj.auxiliaryTracks)) {
              updatedObj.auxiliaryTracks.push(track);
            } else {
              updatedObj.auxiliaryTracks = [track];
            }
          }
          break;
        }
      case HMSTrackUpdate.TRACK_REMOVED:
        {
          const track = data.track;
          if (track.source === HMSTrackSource.REGULAR) {
            if (track.type === HMSTrackType.VIDEO) {
              updatedObj.videoTrack = undefined;
            } else if (track.type === HMSTrackType.AUDIO) {
              updatedObj.audioTrack = undefined;
            }
          } else if (Array.isArray(updatedObj.auxiliaryTracks)) {
            updatedObj.auxiliaryTracks = updatedObj.auxiliaryTracks.filter(auxiliaryTrack => auxiliaryTrack.trackId !== track.trackId);
          }
          break;
        }
      case HMSTrackUpdate.TRACK_MUTED:
      case HMSTrackUpdate.TRACK_UNMUTED:
        {
          const track = data.track;
          if (track.source === HMSTrackSource.REGULAR) {
            if (track.type === HMSTrackType.VIDEO) {
              var _updatedObj$videoTrac;
              const oldIsDegraded = ((_updatedObj$videoTrac = updatedObj.videoTrack) === null || _updatedObj$videoTrac === void 0 ? void 0 : _updatedObj$videoTrac.isDegraded) || false;
              updatedObj.videoTrack = track;
              updatedObj.videoTrack.isDegraded = oldIsDegraded;
            } else if (track.type === HMSTrackType.AUDIO) {
              updatedObj.audioTrack = track;
            }
          } else {
            if (Array.isArray(updatedObj.auxiliaryTracks)) {
              updatedObj = {
                ...updatedObj,
                auxiliaryTracks: updatedObj.auxiliaryTracks.map(auxiliaryTrack => auxiliaryTrack.trackId === track.trackId ? track : auxiliaryTrack)
              };
            } else {
              updatedObj.auxiliaryTracks = [track];
            }
          }
          break;
        }
      case HMSTrackUpdate.TRACK_DEGRADED:
      case HMSTrackUpdate.TRACK_RESTORED:
        {
          const track = data.track;
          if (track.source === HMSTrackSource.REGULAR) {
            if (track.type === HMSTrackType.VIDEO) {
              updatedObj.videoTrack = track;
              updatedObj.videoTrack.isDegraded = updateType === HMSTrackUpdate.TRACK_DEGRADED;
            } else if (track.type === HMSTrackType.AUDIO) {
              updatedObj.audioTrack = track;
            }
          } else {
            if (Array.isArray(updatedObj.auxiliaryTracks)) {
              updatedObj.auxiliaryTracks = updatedObj.auxiliaryTracks.map(auxiliaryTrack => {
                if (auxiliaryTrack.trackId === track.trackId) {
                  return {
                    ...auxiliaryTrack,
                    isDegraded: updateType === HMSTrackUpdate.TRACK_DEGRADED
                  };
                }
                return auxiliaryTrack;
              });
            } else {
              updatedObj.auxiliaryTracks = [track];
            }
          }
          break;
        }
      case HMSPeerUpdate.ROLE_CHANGED:
        {
          updatedObj.role = HMSEncoder.encodeHmsRole(data.role);
          break;
        }
      case HMSPeerUpdate.NETWORK_QUALITY_UPDATED:
        {
          updatedObj.networkQuality = HMSEncoder.encodeHMSNetworkQuality(data.networkQuality);
          break;
        }
      case HMSPeerUpdate.METADATA_CHANGED:
        {
          updatedObj.metadata = data.metadata;
          break;
        }
      case HMSPeerUpdate.NAME_CHANGED:
        {
          updatedObj.name = data.name;
          break;
        }
      case HMSPeerUpdate.HAND_RAISED_CHANGED:
        {
          updatedObj.isHandRaised = data.isHandRaised;
          break;
        }
      default:
        {
          updatedObj = {
            ...updatedObj,
            ...data
          };
          break;
        }
    }
    if (Object.keys(updatedObj).length > 0) {
      this._data.set(peerId, updatedObj);
    }
  }
  cleanup() {
    this._data.clear();
  }
}
export function getPeerPropertyFromNative(id, peerId, property) {
  const data = HMSManager.getPeerProperty({
    id,
    peerId,
    property
  });
  let value;
  if (property === 'role') {
    value = data ? HMSEncoder.encodeHmsRole(data.role) : undefined;
  } else if (property === 'networkQuality') {
    value = data ? HMSEncoder.encodeHMSNetworkQuality(data.networkQuality) : undefined;
  } else if (property === 'audioTrack') {
    value = data ? HMSEncoder.encodeHmsAudioTrack(data.audioTrack, id) : undefined;
  } else if (property === 'videoTrack') {
    value = data ? HMSEncoder.encodeHmsVideoTrack(data.videoTrack, id) : undefined;
  } else if (property === 'auxiliaryTracks') {
    value = data && Array.isArray(data.auxiliaryTracks) ? HMSEncoder.encodeHmsAuxiliaryTracks(data.auxiliaryTracks, id) : undefined;
  } else if (property === 'name') {
    value = data === null || data === void 0 ? void 0 : data[property];
  } else if (property === 'isHandRaised') {
    value = data ? data[property] : false;
  } else {
    value = data ? data[property] : undefined;
  }
  return value;
}
//# sourceMappingURL=HMSPeersCache.js.map