function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == typeof i ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != typeof t || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != typeof i) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
import HMSManager from '../modules/HMSManagerModule';
import { HMSEncoder } from './HMSEncoder';
import { HMSRoomUpdate } from './HMSRoomUpdate';
export let hmsRoomCache;
export const getHmsRoomCache = () => hmsRoomCache;
export const setHmsRoomCache = _hmsRoomCache => {
  hmsRoomCache = _hmsRoomCache;
};
export const clearHmsRoomCache = () => {
  if (hmsRoomCache) {
    hmsRoomCache.cleanup();
  }
  hmsRoomCache = undefined;
};
export class HMSRoomCache {
  constructor(id) {
    _defineProperty(this, "_data", new Map());
    _defineProperty(this, "id", void 0);
    this.id = id;
  }
  getProperty(property) {
    const value = this._data.get(property);
    if (value) {
      return value;
    }
    const encodedValue = getHMSRoomPropertyFromNative(this.id, property);
    this._data.set(property, encodedValue);
    return encodedValue;
  }
  updateRoomCache(data, updateType) {
    switch (updateType) {
      case HMSRoomUpdate.ROOM_MUTED:
      case HMSRoomUpdate.ROOM_UNMUTED:
        {
          break;
        }
      case HMSRoomUpdate.SERVER_RECORDING_STATE_UPDATED:
        {
          this._data.set('serverRecordingState', HMSEncoder.encodeServerRecordingState(data.serverRecordingState));
          break;
        }
      case HMSRoomUpdate.RTMP_STREAMING_STATE_UPDATED:
        {
          this._data.set('rtmpHMSRtmpStreamingState', HMSEncoder.encodeRTMPStreamingState(data.rtmpHMSRtmpStreamingState));
          break;
        }
      case HMSRoomUpdate.BROWSER_RECORDING_STATE_UPDATED:
        {
          this._data.set('browserRecordingState', HMSEncoder.encodeBrowserRecordingState(data.browserRecordingState));
          break;
        }
      case HMSRoomUpdate.HLS_STREAMING_STATE_UPDATED:
        {
          this._data.set('hlsStreamingState', HMSEncoder.encodeHLSStreamingState(data.hlsStreamingState));
          break;
        }
      case HMSRoomUpdate.HLS_RECORDING_STATE_UPDATED:
        {
          this._data.set('hlsRecordingState', HMSEncoder.encodeHLSRecordingState(data.hlsRecordingState));
          break;
        }
      case HMSRoomUpdate.ROOM_PEER_COUNT_UPDATED:
        {
          this._data.set('peerCount', encodeHMSRoomProperty(this.id, 'peerCount', data));
          break;
        }
      case HMSRoomUpdate.TRANSCRIPTIONS_UPDATED:
        {
          this._data.set('transcriptions', encodeHMSRoomProperty(this.id, 'transcriptions', data));
          break;
        }
      default:
        {
          for (const key in data) {
            const property = key;
            if (Object.prototype.hasOwnProperty.call(data, property)) {
              const value = encodeHMSRoomProperty(this.id, property, data);
              this._data.set(property, value);
            }
          }
          break;
        }
    }
  }
  cleanup() {
    this._data.clear();
  }
}
export function getHMSRoomPropertyFromNative(id, property) {
  const data = HMSManager.getRoomProperty({
    id,
    property
  });
  if (!data) return data;
  const value = encodeHMSRoomProperty(id, property, data);
  return value;
}
function encodeHMSRoomProperty(id, property, data) {
  let value;
  if (property === 'peers') {
    value = HMSEncoder.encodeHmsPeers(data.peers);
  } else if (property === 'browserRecordingState') {
    value = HMSEncoder.encodeBrowserRecordingState(data.browserRecordingState);
  } else if (property === 'rtmpHMSRtmpStreamingState') {
    value = HMSEncoder.encodeRTMPStreamingState(data.rtmpHMSRtmpStreamingState);
  } else if (property === 'serverRecordingState') {
    value = HMSEncoder.encodeServerRecordingState(data.serverRecordingState);
  } else if (property === 'hlsStreamingState') {
    value = HMSEncoder.encodeHLSStreamingState(data.hlsStreamingState);
  } else if (property === 'hlsRecordingState') {
    value = HMSEncoder.encodeHLSRecordingState(data.hlsRecordingState);
  } else if (property === 'transcriptions') {
    value = HMSEncoder.encodeTranscriptions(data.transcriptions);
  } else if (property === 'localPeer') {
    value = HMSEncoder.encodeHmsLocalPeer(data.localPeer, id);
  } else if (property === 'peerCount') {
    value = typeof data[property] === 'number' ? data[property] : null;
  } else {
    value = data[property];
  }
  return value;
}
//# sourceMappingURL=HMSRoomCache.js.map