import type { HMSPoll } from './polls/HMSPoll';
import type { HMSPollCreateParams } from './polls/HMSPollCreateParams';
import type { HMSPollUpdateType } from './polls/HMSPollUpdateType';
import type { PollLeaderboardResponse } from './polls/PollLeaderboardResponse';
import type { HMSWhiteboardUpdateType } from './whiteboard/HMSWhiteboardUpdateType';
import type { HMSWhiteboard } from './whiteboard/HMSWhiteboard';
type PollQuestionResponseCreateParams = {
    pollId: string;
    pollQuestionIndex: number;
    responses: {
        text: string;
        duration?: number;
    } | {
        options: number[];
        duration?: number;
    };
};
export declare class HMSInteractivityCenter {
    private _eventEmitter;
    private onUpdateListenerRemoved;
    constructor();
    private _pollUpdateListener;
    /**
     * Adds a listener for poll updates
     * @param listener - Callback to be called when a poll is updated
     * @returns HMSEventSubscription
     */
    addPollUpdateListener(listener: (updatedPoll: HMSPoll, update: HMSPollUpdateType) => void): import("../utils").EmitterSubscription;
    /**
     * Starts a poll
     *
     * @param pollParams
     */
    startPoll(pollParams: HMSPollCreateParams): Promise<any>;
    add(pollQuestionResponseParams: PollQuestionResponseCreateParams): Promise<any>;
    stop(pollId: string): Promise<any>;
    /**
     * Fetches the leaderboard for a poll
     * @param pollId - The id of the poll
     * @param count - The number of entries to fetch
     * @param startIndex - The index to start fetching from
     * @param includeCurrentPeer - Whether to include the current peer in the fetched leaderboard entries
     * @returns Promise<PollLeaderboardResponse>
     */
    fetchLeaderboard(pollId: string, count: number, startIndex: number, includeCurrentPeer: boolean): Promise<PollLeaderboardResponse>;
    private _whiteboardUpdateListener;
    /**
     * Adds a listener for Whiteboard updates
     * @param listener - Callback to be called when whiteboard is updated
     * @returns HMSEventSubscription
     */
    addWhiteboardUpdateListener(listener: (hmsWhiteboard: HMSWhiteboard, updateType: HMSWhiteboardUpdateType) => void): import("../utils").EmitterSubscription;
    /**
     * Starts Whiteboard
     */
    startWhiteboard(title: string): Promise<any>;
    /**
     * Stops Whiteboard
     */
    stopWhiteboard(): Promise<any>;
}
export {};
