import React from 'react';
import type { ViewStyle } from 'react-native';
import { HMSVideoViewMode } from './HMSVideoViewMode';
/**
 * Defines the properties for the `HmsViewComponent`.
 *
 * This interface outlines the props that can be passed to the `HmsViewComponent` to configure its behavior and appearance.
 *
 * @interface HmsComponentProps
 * @property {string} trackId - The unique identifier for the track to be displayed.
 * @property {ViewStyle} [style] - Optional. Custom styles to apply to the view.
 * @property {boolean} [mirror] - Optional. If true, the video will be mirrored. This is commonly used for local video tracks.
 * @property {boolean} [autoSimulcast] - Optional. Enables automatic simulcast layer switching based on network conditions, if supported.
 * @property {HMSVideoViewMode} [scaleType] - Optional. Determines how the video fits within the bounds of the view (e.g., aspect fill, aspect fit).
 * @property {boolean} [setZOrderMediaOverlay] - Optional. When true, the video view will be rendered above the regular view hierarchy.
 * @property {string} id - The identifier for the `HmsViewComponent` instance.
 *
 * @see {https://www.100ms.live/docs/react-native/v2/how-to-guides/set-up-video-conferencing/render-video/overview}
 */
export interface HmsComponentProps {
    trackId: string;
    style?: ViewStyle;
    mirror?: boolean;
    autoSimulcast?: boolean;
    scaleType?: HMSVideoViewMode;
    setZOrderMediaOverlay?: boolean;
    id: string;
}
export declare const HmsViewComponent: React.ForwardRefExoticComponent<HmsComponentProps & React.RefAttributes<any>>;
